

/* this file was generated by volk template utils, do not edit! */

/*
 * Copyright 2011-2012 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_VOLK_CONFIG_FIXED_H
#define INCLUDED_VOLK_CONFIG_FIXED_H

#define LV_GENERIC 0
#define LV_SOFTFP 1
#define LV_HARDFP 2
#define LV_NEON 3
#define LV_NEONV7 4
#define LV_NEONV8 5
#define LV_32 6
#define LV_64 7
#define LV_3DNOW 8
#define LV_ABM 9
#define LV_POPCOUNT 10
#define LV_MMX 11
#define LV_FMA 12
#define LV_SSE 13
#define LV_SSE2 14
#define LV_ORC 15
#define LV_NORC 16
#define LV_SSE3 17
#define LV_SSSE3 18
#define LV_SSE4_A 19
#define LV_SSE4_1 20
#define LV_SSE4_2 21
#define LV_AVX 22
#define LV_AVX2 23
#define LV_AVX512F 24
#define LV_AVX512CD 25

#endif /*INCLUDED_VOLK_CONFIG_FIXED*/
