



where volk-config-info
IF %ERRORLEVEL% NEQ 0 exit /B 1
where volk_profile
IF %ERRORLEVEL% NEQ 0 exit /B 1
where volk_modtool.py
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\bin\\volk.dll exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\volk.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
volk-config-info --version --prefix --cc --cflags --all-machines --avail-machines --machine --alignment --malloc
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
